@@ -1,4 +1,3 @@
-from django.db.models import Sum, Q
 from django.test import TestCase, override_settings
 from wagtail.core.models import Page
 
@@ -20,7 +19,25 @@ class TestServe(TestCase):
             status=AbTest.STATUS_RUNNING,
         )
 
-    def test_serves_control(self):
+    def test_serves_control_from_cookie(self):
+        self.client.cookies[f'wagtail-ab-testing_{self.ab_test.id}_version'] = AbTest.VERSION_CONTROL
+
+        response = self.client.get('/')
+        self.assertEqual(response.status_code, 200)
+
+        self.assertContains(response, "Welcome to your new Wagtail site!")
+        self.assertNotContains(response, "Changed title")
+
+    def test_serves_variant_from_cookie(self):
+        self.client.cookies[f'wagtail-ab-testing_{self.ab_test.id}_version'] = AbTest.VERSION_VARIANT
+
+        response = self.client.get('/')
+        self.assertEqual(response.status_code, 200)
+
+        self.assertNotContains(response, "Welcome to your new Wagtail site!")
+        self.assertContains(response, "Changed title")
+
+    def test_serves_control_to_new_participant(self):
         # Add a participant for variant
         # This will make the new participant use control to balance the numbers
         self.ab_test.add_participant(AbTest.VERSION_VARIANT)
@@ -31,9 +48,7 @@ class TestServe(TestCase):
         self.assertContains(response, "Welcome to your new Wagtail site!")
         self.assertNotContains(response, "Changed title")
 
-        self.assertEqual(self.client.session[f'wagtail-ab-testing_{self.ab_test.id}_version'], AbTest.VERSION_CONTROL)
-
-    def test_serves_variant(self):
+    def test_serves_variant_to_new_participant(self):
         # Add a participant for control
         # This will make the new participant use variant to balance the numbers
         self.ab_test.add_participant(AbTest.VERSION_CONTROL)
@@ -44,8 +59,6 @@ class TestServe(TestCase):
         self.assertNotContains(response, "Welcome to your new Wagtail site!")
         self.assertContains(response, "Changed title")
 
-        self.assertEqual(self.client.session[f'wagtail-ab-testing_{self.ab_test.id}_version'], AbTest.VERSION_VARIANT)
-
     def test_serves_control_when_paused(self):
         self.ab_test.status = AbTest.STATUS_PAUSED
         self.ab_test.save()
@@ -62,20 +75,6 @@ class TestServe(TestCase):
 
         self.assertNotIn(f'wagtail-ab-testing_{self.ab_test.id}_version', self.client.session)
 
-    @override_settings(WAGTAIL_AB_TESTING={'MODE': 'external'})
-    def test_serves_control_when_in_external_mode(self):
-        # Add a participant for control
-        # This time, the next viewer will still see the control as the test is not running
-        self.ab_test.add_participant(AbTest.VERSION_CONTROL)
-
-        response = self.client.get('/')
-        self.assertEqual(response.status_code, 200)
-
-        self.assertContains(response, "Welcome to your new Wagtail site!")
-        self.assertNotContains(response, "Changed title")
-
-        self.assertNotIn(f'wagtail-ab-testing_{self.ab_test.id}_version', self.client.session)
-
     def test_doesnt_track_bots(self):
         # Add a participant for control
         # This will make it serve the variant if it does incorrectly decide to track the user
@@ -105,51 +104,24 @@ class TestServe(TestCase):
         self.assertNotContains(response, "Changed title")
         self.assertNotIn(f'wagtail-ab-testing_{self.ab_test.id}_version', self.client.session)
 
-    def test_visit_page_goal_completion(self):
-        session = self.client.session
-        session[f'wagtail-ab-testing_{self.ab_test.id}_version'] = 'variant'
-        session.save()
-
-        response = self.client.get('/goal/')
+    @override_settings(WAGTAIL_AB_TESTING_WORKER_TOKEN='abc123')
+    def test_serves_dual_response_for_worker(self):
+        response = self.client.get('/', HTTP_X_REQUESTED_WITH='WagtailAbTestingWorker', HTTP_AUTHORIZATION='Token abc123')
         self.assertEqual(response.status_code, 200)
 
-        self.assertEqual(self.client.session[f'wagtail-ab-testing_{self.ab_test.id}_completed'], 'yes')
+        self.assertEqual(response['X-WagtailAbTesting-Test'], str(self.ab_test.id))
+        response_data = response.json()
 
-        stats = self.ab_test.hourly_logs.aggregate(
-            control_conversions=Sum('conversions', filter=Q(version=AbTest.VERSION_CONTROL)),
-            variant_conversions=Sum('conversions', filter=Q(version=AbTest.VERSION_VARIANT)),
-        )
+        self.assertIn("Welcome to your new Wagtail site!", response_data['control'])
+        self.assertIn("Changed title", response_data['variant'])
 
-        self.assertEqual(stats['control_conversions'] or 0, 0)
-        self.assertEqual(stats['variant_conversions'] or 0, 1)
+    @override_settings(WAGTAIL_AB_TESTING_WORKER_TOKEN='abc123')
+    def test_worker_requires_token(self):
+        response = self.client.get('/', HTTP_X_REQUESTED_WITH='WagtailAbTestingWorker')
 
-    def test_visit_page_goal_completion_doesnt_count_second_time(self):
-        # Shouldn't be counted if test marked as completed in users session
-        session = self.client.session
-        session[f'wagtail-ab-testing_{self.ab_test.id}_version'] = 'variant'
-        session[f'wagtail-ab-testing_{self.ab_test.id}_completed'] = 'yes'
-        session.save()
+        self.assertEqual(response.status_code, 403)
 
-        response = self.client.get('/goal/')
-        self.assertEqual(response.status_code, 200)
-
-        stats = self.ab_test.hourly_logs.aggregate(
-            control_conversions=Sum('conversions', filter=Q(version=AbTest.VERSION_CONTROL)),
-            variant_conversions=Sum('conversions', filter=Q(version=AbTest.VERSION_VARIANT)),
-        )
-
-        self.assertEqual(stats['control_conversions'] or 0, 0)
-        self.assertEqual(stats['variant_conversions'] or 0, 0)
-
-    def test_visit_page_goal_completion_doesnt_count_if_not_participant(self):
-        # Shouldn't be counted if the user is not a participant
-        response = self.client.get('/goal/')
-        self.assertEqual(response.status_code, 200)
-
-        stats = self.ab_test.hourly_logs.aggregate(
-            control_conversions=Sum('conversions', filter=Q(version=AbTest.VERSION_CONTROL)),
-            variant_conversions=Sum('conversions', filter=Q(version=AbTest.VERSION_VARIANT)),
-        )
-
-        self.assertEqual(stats['control_conversions'] or 0, 0)
-        self.assertEqual(stats['variant_conversions'] or 0, 0)
+    @override_settings(WAGTAIL_AB_TESTING_WORKER_TOKEN='abc123')
+    def test_worker_requires_correct_token(self):
+        response = self.client.get('/', HTTP_X_REQUESTED_WITH='WagtailAbTestingWorker', HTTP_AUTHORIZATION='Token wrongtoken')
+        self.assertEqual(response.status_code, 403)
