@@ -1,6 +1,9 @@
 import json
 
+from django.conf import settings
 from django.contrib.auth.models import Permission
+from django.core.exceptions import PermissionDenied
+from django.http import JsonResponse
 from django.shortcuts import redirect
 from django.urls import path, include, reverse
 from django.utils.html import format_html, escapejs
@@ -14,7 +17,6 @@ from wagtail.core import hooks
 
 from . import views
 from .compat import DATE_FORMAT
-from .conf import get_conf
 from .models import AbTest
 from .utils import request_is_trackable
 
@@ -34,8 +36,8 @@ def register_admin_urls():
         path(
             "abtests/",
             include(
-                (urls, "wagtail_ab_testing"),
-                namespace="wagtail_ab_testing",
+                (urls, "wagtail_ab_testing_admin"),
+                namespace="wagtail_ab_testing_admin",
             ),
         )
     ]
@@ -68,7 +70,7 @@ class AbTestingTabActionMenuItem(ActionMenuItem):
         if 'page' in context:
             return format_html(
                 '<script src="{}"></script><script src="{}"></script><script>window.abTestingTabProps = JSON.parse("{}");</script>',
-                reverse('wagtail_ab_testing:javascript_catalog'),
+                reverse('wagtail_ab_testing_admin:javascript_catalog'),
                 versioned_static('wagtail_ab_testing/js/wagtail-ab-testing.js'),
                 escapejs(json.dumps({
                     'tests': [
@@ -77,7 +79,7 @@ class AbTestingTabActionMenuItem(ActionMenuItem):
                             'name': ab_test.name,
                             'started_at': ab_test.first_started_at.strftime(DATE_FORMAT) if ab_test.first_started_at else _("Not started"),
                             'status': ab_test.get_status_description(),
-                            'results_url': reverse('wagtail_ab_testing:results', args=[ab_test.page_id, ab_test.id]),
+                            'results_url': reverse('wagtail_ab_testing_admin:results', args=[ab_test.page_id, ab_test.id]),
                         }
                         for ab_test in AbTest.objects.filter(page=context['page']).order_by('-id')
                     ],
@@ -96,7 +98,7 @@ def register_ab_testing_tab_action_menu_item():
 @hooks.register('after_edit_page')
 def redirect_to_create_ab_test(request, page):
     if 'create-ab-test' in request.POST:
-        return redirect('wagtail_ab_testing:add_ab_test_compare', page.id)
+        return redirect('wagtail_ab_testing_admin:add_ab_test_compare', page.id)
 
 
 @hooks.register('before_edit_page')
@@ -108,41 +110,58 @@ def check_for_running_ab_test(request, page):
 
 @hooks.register('before_serve_page')
 def before_serve_page(page, request, serve_args, serve_kwargs):
-    # Check if we're running in 'internal' mode
-    if not get_conf()['MODE'] == 'internal':
-        return
-
     # Check if the user is trackable
     if not request_is_trackable(request):
         return
 
-    # Check if visiting the page is the goal of any running tests
-    tests = AbTest.objects.filter(goal_event='visit-page', goal_page=page, status=AbTest.STATUS_RUNNING)
-    for test in tests:
-        # Is the user a participant in this test?
-        if f'wagtail-ab-testing_{test.id}_version' not in request.session:
-            continue
-
-        # Has the user already completed the test?
-        if f'wagtail-ab-testing_{test.id}_completed' in request.session:
-            continue
-
-        # Log a conversion
-        test.log_conversion(request.session[f'wagtail-ab-testing_{test.id}_version'])
-        request.session[f'wagtail-ab-testing_{test.id}_completed'] = 'yes'
-
-    # Check if the page itself is running any tests
+    # Check for a running A/B test on the requested page
     try:
         test = AbTest.objects.get(page=page, status=AbTest.STATUS_RUNNING)
     except AbTest.DoesNotExist:
         return
 
-    # Make the user a participant if they're not already
-    if f'wagtail-ab-testing_{test.id}_version' not in request.session:
-        request.session[f'wagtail-ab-testing_{test.id}_version'] = test.add_participant()
+    # Save reference to test on request object so it can be found by the {% wagtail_ab_testing_script %} template tag
+    request.wagtail_ab_testing_test = test
 
-    # If the user is visiting the variant version, serve that from the revision
-    if request.session[f'wagtail-ab-testing_{test.id}_version'] == AbTest.VERSION_VARIANT:
+    # If this request is coming from a frontend worker, return both the control and variant versions
+    # The worker will decide which version to serve to the user
+    if request.META.get('HTTP_X_REQUESTED_WITH') == 'WagtailAbTestingWorker':
+        if request.META.get('HTTP_AUTHORIZATION', '') != 'Token ' + settings.WAGTAIL_AB_TESTING_WORKER_TOKEN:
+            raise PermissionDenied
+
+        control_response = page.serve(request, *serve_args, **serve_kwargs)
+
+        # Note: we must render the control response before setting `wagtail_ab_testing_serving_variant`
+        if hasattr(control_response, "render"):
+            control_response.render()
+
+        request.wagtail_ab_testing_serving_variant = True
+
+        variant_response = test.variant_revision.as_page_object().serve(request, *serve_args, **serve_kwargs)
+        if hasattr(variant_response, "render"):
+            variant_response.render()
+
+        response = JsonResponse({
+            'control': control_response.content.decode('utf-8'),
+            'variant': variant_response.content.decode('utf-8'),
+        })
+
+        response['X-WagtailAbTesting-Test'] = str(test.id)
+
+        return response
+
+    # If the user visiting is a participant, show them the same version they saw before
+    if f'wagtail-ab-testing_{test.id}_version' in request.COOKIES:
+        version = request.COOKIES[f'wagtail-ab-testing_{test.id}_version']
+    else:
+        # Otherwise, show them the version of the page that the next participant should see.
+        # Note: In order to exclude bots, the browser must call a JavaScript API to sign up as a participant
+        # Once they've signed up, they'll get a cookie which keeps them on the same version
+        version = test.get_new_participant_version()
+
+    # If the user should be shown the variant, serve that from the revision. Otherwise return to keep the control
+    if version == AbTest.VERSION_VARIANT:
+        request.wagtail_ab_testing_serving_variant = True
         return test.variant_revision.as_page_object().serve(request, *serve_args, **serve_kwargs)
 
 
@@ -154,7 +173,7 @@ class AbTestingReportMenuItem(MenuItem):
 
 @hooks.register('register_reports_menu_item')
 def register_ab_testing_report_menu_item():
-    return AbTestingReportMenuItem(_('A/B testing'), reverse('wagtail_ab_testing:report'), icon_name='people-arrows', order=1000)
+    return AbTestingReportMenuItem(_('A/B testing'), reverse('wagtail_ab_testing_admin:report'), icon_name='people-arrows', order=1000)
 
 
 @hooks.register('register_icons')
